@@ -1,13 +1,14 @@
 from typing import Dict, List
 import numpy as np
 import pandas as pd
+import scipy as sp
 import logging
 import plotly.graph_objects as go
 
 from .alignment_helpers import distance_matrix, backtrack_optimal_path
-from .data_input import dict_to_wide
+from .data_input import dict_to_wide, check_valid_batch_dict
 
-from ..multivariate import MCUVScaler, PCA
+from ..multivariate.methods import MCUVScaler, PCA
 
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
@@ -81,7 +82,6 @@ def determine_scaling(
             axis=1,
         )
     scalings.columns = ["Range", "Minimum"]
-    # scalings["Minimum"] = 0.0
     return scalings
 
 
@@ -329,11 +329,7 @@ def batch_dtw(
     ), "`reference_batch` was not found in the dict of batches."
     settings["subsample"] = int(settings["subsample"])
 
-    # Checks on the batch data (put in a sub-function)
-    # TODO assert False, "Is this actually necessary? Why do this?"
-    for batch_id, batch in batches.items():
-        if "batch_id" not in batch.columns:
-            batches[batch_id].insert(0, "batch_id", batch_id)
+    assert check_valid_batch_dict(batches, no_nan=True)
 
     scale_df = determine_scaling(
         batches=batches, columns_to_align=columns_to_align, settings=settings
@@ -398,17 +394,10 @@ def batch_dtw(
     aligned_df = pd.DataFrame()
 
     for batch_id, result in aligned_batches.items():
-        initial_row = (
-            batches[batch_id]
-            .drop("batch_id", axis=1)
-            .iloc[result.md_path[0, 0], :]
-            .copy()
-        )
+        initial_row = batches[batch_id].iloc[result.md_path[0, 0], :].copy()
         synced = align_with_path(
             result.md_path,
-            batches[batch_id]
-            .drop("batch_id", axis=1)
-            .iloc[:: int(settings["subsample"]), :],
+            batches[batch_id].iloc[:: int(settings["subsample"]), :],
             initial_row=initial_row,
         )
         if "batch_id" not in synced.columns:
@@ -467,8 +456,32 @@ def resample_to_reference(
     dict
         Batch data, in the standard format.
     """
-    # TODO
-    pass
+    default_settings = {
+        "interpolate_kind": "cubic",  # must be a valid "scipy.interpolate.interp1d" `kind`
+    }
+    if settings:
+        default_settings.update(settings)
+    settings = default_settings
+    out = {}
+    target_time = np.arange(0, batches[reference_batch].shape[0])
+    target_time = target_time / target_time[-1]
+
+    for batch_id, batch in batches.items():
+        to_resample = np.arange(0, batch.shape[0])
+        to_resample = to_resample / to_resample[-1]
+        out_df = {}
+        for column, series in batch.iteritems():
+            if column in columns_to_align:
+                out_df[column] = sp.interpolate.interp1d(
+                    to_resample,
+                    series.values,
+                    copy=False,
+                    kind=settings["interpolate_kind"],
+                )(target_time)
+
+        out[batch_id] = pd.DataFrame(out_df)
+
+    return out
 
 
 def find_average_length(batches: Dict[str, pd.DataFrame], settings: dict = None):
@@ -548,12 +561,15 @@ def find_reference_batch(
         "robust": True,  # use robust scaling
         "subsample": 1,  # use every sample
         "method": "pca_most_average",
+        "n_components": 4,
     }
-    if settings:
+    if isinstance(settings, dict):
         default_settings.update(settings)
     settings = default_settings
     settings["subsample"] = int(settings["subsample"])
 
+    assert check_valid_batch_dict(batches)
+
     # Starts with the average duration batch.
     initial_reference_id = find_average_length(batches, settings)
 
@@ -575,8 +591,22 @@ def find_reference_batch(
     pca_first = PCA(n_components=settings["n_components"]).fit(mcuv)
 
     # Excludes all batches with Hotelling's T2 > 90% limit.
-    _ = pca_first.T2_limit(0.90)  # = T2_limit_90
+    T2_limit_90 = pca_first.T2_limit(0.90)
+    to_keep = pca_first.Hotellings_T2.iloc[:, -1] < T2_limit_90
 
-    # Refits PCA with A=4
+    # Refits PCA with A=4 on a subset of the batches, to avoid biasing the PCA model too much.
+    basewide = basewide.loc[to_keep, :]
+    scaler = MCUVScaler().fit(basewide)
+    mcuv = scaler.fit_transform(basewide)
+    pca_second = PCA(n_components=settings["n_components"]).fit(mcuv)
 
     # Finds batch with scores; and ensures this batch has SPE < 50% of the model limit.
+    metrics = pd.DataFrame(
+        {
+            "HT2": pca_second.Hotellings_T2.iloc[:, -1],
+            "SPE": pca_second.squared_prediction_error.iloc[:, -1],
+        }
+    )
+    metrics = metrics.sort_values(by=["HT2", "SPE"])
+    metrics = metrics.query(f"SPE < {pca_second.SPE_limit(0.5)}")
+    return metrics.index[0]
