@@ -9,7 +9,7 @@ import pandas as pd
 import numpy as np
 
 from sklearn.cross_decomposition import PLSRegression
-from process_improve.multivariate import (
+from process_improve.multivariate.methods import (
     epsqrt,
     center,
     scale,
